@@ -13,9 +13,8 @@
  * layer information is already known.  The DEF file should have information
  * primarily on die are, track placement, pins, components, and nets.
  *
- * To-do: Routed nets should have their routes dropped into track obstructions,
- * and the nets should be ignored.  Currently, routed nets are parsed and the
- * routes are ignored.
+ * Routed nets have their routes dropped into track obstructions, and the
+ * nets are ignored.
  */
 
 #include <stdio.h>
@@ -33,6 +32,8 @@
 #include "lef.h"
 #include "def.h"
 
+int numSpecial = 0;		/* Tracks number of specialnets */
+
 #ifndef TCL_QROUTER
 
 /* Find an instance in the instance list.  If qrouter	*/
@@ -146,7 +147,7 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
     char valid = FALSE;		/* is there a valid reference point? */
     char initial = TRUE;
     struct dseg_ locarea;
-    double x, y, lx, ly, w;
+    double x, y, lx, ly, w, hw, s;
     int routeLayer = -1, paintLayer;
     LefList lefl;
     ROUTE routednet = NULL;
@@ -205,6 +206,8 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 	       routednet->netnum = net->netnum;
 	       routednet->segments = NULL;
 	       routednet->flags = (u_char)0;
+	       routednet->start.route = NULL;
+	       routednet->end.route = NULL;
 	    }
 	}
 	else if (*token != '(')	/* via name */
@@ -234,21 +237,25 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 			   if (routeLayer < paintLayer) paintLayer = routeLayer;
 			   if ((routeLayer >= 0) && (special == (char)1) &&
 					(valid == TRUE)) {
-			      drect = (DSEG)malloc(sizeof(struct dseg_));
-			      drect->x1 = x + lr->x1;
-			      drect->x2 = x + lr->x2;
-			      drect->y1 = y + lr->y1;
-			      drect->y2 = y + lr->y2;
-			      drect->layer = lr->layer;
-			      drect->next = UserObs;
-			      UserObs = drect;
+				s = LefGetRouteSpacing(routeLayer); 
+				drect = (DSEG)malloc(sizeof(struct dseg_));
+				drect->x1 = x + lr->x1 - s;
+				drect->x2 = x + lr->x2 + s;
+				drect->y1 = y + lr->y1 - s;
+				drect->y2 = y + lr->y2 + s;
+				drect->layer = routeLayer;
+				drect->next = UserObs;
+				UserObs = drect;
 			   }
 			   lr = lr->next;
 			}
 			if (routeLayer == -1) paintLayer = lefl->type;
 		    }
-		    else
+		    else {
 		    	paintLayer = lefl->type;
+			if (special == (char)1)
+			    s = LefGetRouteSpacing(paintLayer); 
+		    }
 		}
 		else
 		{
@@ -273,6 +280,8 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 			routednet->netnum = net->netnum;
 			routednet->segments = NULL;
 			routednet->flags = (u_char)0;
+			routednet->start.route = NULL;
+			routednet->end.route = NULL;
 		    }
 		    newRoute->next = routednet->segments;
 		    routednet->segments = newRoute;
@@ -359,27 +368,35 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 	    {
 		locarea.x2 = refp.x1;
 		locarea.y2 = refp.y1;
-		lx = x;
-		ly = y;
 
 		if (special == (char)1) {
 		   if (valid == TRUE) {
+		      s = LefGetRouteSpacing(routeLayer); 
+		      hw = w / 2;
 		      drect = (DSEG)malloc(sizeof(struct dseg_));
 		      if (lx > x) {
-		         drect->x1 = x - w;
-		         drect->x2 = lx + w;
+		         drect->x1 = x - s;
+		         drect->x2 = lx + s;
+		      }
+		      else if (lx < x) {
+		         drect->x1 = lx - s;
+		         drect->x2 = x + s;
 		      }
 		      else {
-		         drect->x1 = x + w;
-		         drect->x2 = lx - w;
+		         drect->x1 = x - hw - s;
+		         drect->x2 = x + hw + s;
 		      }
 		      if (ly > y) {
-		         drect->y1 = y - w;
-		         drect->y2 = ly + w;
+		         drect->y1 = y - s;
+		         drect->y2 = ly + s;
+		      }
+		      else if (ly < y) {
+		         drect->y1 = ly - s;
+		         drect->y2 = y + s;
 		      }
 		      else {
-		         drect->y1 = y + w;
-		         drect->y2 = ly - w;
+		         drect->y1 = y - hw - s;
+		         drect->y2 = y + hw + s;
 		      }
 		      drect->layer = routeLayer;
 		      drect->next = UserObs;
@@ -403,6 +420,8 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 			routednet->netnum = net->netnum;
 			routednet->segments = NULL;
 			routednet->flags = (u_char)0;
+			routednet->start.route = NULL;
+			routednet->end.route = NULL;
 		   }
 		   newRoute->next = routednet->segments;
 		   routednet->segments = newRoute;
@@ -478,22 +497,23 @@ DefReadGatePin(NET net, NODE node, char *instname, char *pinname, double *home)
 
 		    gridx = (int)((drect->x1 - Xlowerbound) /
 				PitchX[drect->layer]) - 1;
+
 		    if (gridx < 0) gridx = 0;
 		    while (1) {
 			dx = (gridx * PitchX[drect->layer]) + Xlowerbound;
-			if (dx > drect->x2 + home[drect->layer]) break;
-			if (dx < drect->x1 - home[drect->layer]) {
+			if (dx > drect->x2 + home[drect->layer] - EPS) break;
+			if (dx < drect->x1 - home[drect->layer] + EPS) {
 			    gridx++;
 			    continue;
 			}
 			gridy = (int)((drect->y1 - Ylowerbound) /
 					PitchY[drect->layer]) - 1;
+
 			if (gridy < 0) gridy = 0;
 			while (1) {
-			    dy = (gridy * PitchY[drect->layer])
-					+ Ylowerbound;
-			    if (dy > drect->y2 + home[drect->layer]) break;
-			    if (dy < drect->y1 - home[drect->layer]) {
+			    dy = (gridy * PitchY[drect->layer]) + Ylowerbound;
+			    if (dy > drect->y2 + home[drect->layer] - EPS) break;
+			    if (dy < drect->y1 - home[drect->layer] + EPS) {
 				gridy++;
 				continue;
 			    }
@@ -508,8 +528,10 @@ DefReadGatePin(NET net, NODE node, char *instname, char *pinname, double *home)
 			    dp->gridx = gridx;
 			    dp->gridy = gridy;
 
-			    if (dy >= drect->y1 && dx >= drect->x1 &&
-					dy <= drect->y2 && dx <= drect->x2) {
+			    if ((dy >= drect->y1 - EPS) &&
+					(dx >= drect->x1 - EPS) &&
+					(dy <= drect->y2 + EPS) &&
+					(dx <= drect->x2 + EPS)) {
 				dp->next = node->taps;
 				node->taps = dp;
 			    }
@@ -893,6 +915,7 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
     GATE gate;
     int curlayer;
     double hwidth;
+    u_char pin_use;
 
     static char *pin_keys[] = {
 	"-",
@@ -921,6 +944,21 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
 	NULL
     };
 
+    static char *pin_uses[] = {
+	"DEFAULT",
+	"SIGNAL",
+	"ANALOG",
+	"POWER",
+	"GROUND",
+	"CLOCK",
+	"TIEOFF",
+	"SCAN",
+	"RESET",
+	NULL
+    };
+
+    pin_use = PORT_USE_DEFAULT;
+
     while ((token = LefNextToken(f, TRUE)) != NULL)
     {
 	keyword = Lookup(token, pin_keys);
@@ -961,12 +999,14 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
 		/* Pin record has one node;  allocate memory for it */
 		gate->taps = (DSEG *)malloc(sizeof(DSEG));
 		gate->noderec = (NODE *)malloc(sizeof(NODE));
+		gate->direction = (u_char *)malloc(sizeof(u_char));
 		gate->netnum = (int *)malloc(sizeof(int));
 		gate->node = (char **)malloc(sizeof(char *));
 		gate->taps[0] = NULL;
 		gate->noderec[0] = NULL;
 		gate->netnum[0] = -1;
 		gate->node[0] = NULL;
+		gate->direction[0] = PORT_CLASS_DEFAULT;
 
 		/* Now do a search through the line for "+" entries	*/
 		/* And process each.					*/
@@ -996,7 +1036,9 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
 			    token = LefNextToken(f, TRUE);
 			    subkey = Lookup(token, pin_classes);
 			    if (subkey < 0)
-				LefError("Unknown pin class\n");
+				LefError("Unknown pin class %s\n", token);
+			    else
+				gate->direction[0] = subkey;
 			    break;
 			case DEF_PINS_PROP_LAYER:
 			    curlayer = LefReadLayer(f, FALSE);
@@ -1005,7 +1047,12 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
 			    gate->height = currect->y2 - currect->y1;
 			    break;
 			case DEF_PINS_PROP_USE:
-			    /* Ignore this, for now */
+			    token = LefNextToken(f, TRUE);
+			    subkey = Lookup(token, pin_uses);
+			    if (subkey < 0)
+				LefError("Unknown pin use %s\n", token);
+			    else
+				pin_use = subkey;
 			    break;
 			case DEF_PINS_PROP_PLACED:
 			case DEF_PINS_PROP_FIXED:
@@ -1048,6 +1095,7 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
 				pinname);
 		    free(gate->taps);
 		    free(gate->noderec);
+		    free(gate->direction);
 		    free(gate->netnum);
 		    free(gate->node);
 		    free(gate);
@@ -1061,6 +1109,26 @@ DefReadPins(FILE *f, char *sname, float oscale, int total)
 		    LefError("Pins END statement missing.\n");
 		    keyword = -1;
 		}
+		if (pin_use != PORT_USE_DEFAULT && gate->direction[0] ==
+			PORT_CLASS_DEFAULT)
+		{
+		    /* Derive pin use from pin class, if needed */
+		    switch (pin_use) {
+			case PORT_USE_SIGNAL:
+			case PORT_USE_RESET:
+			case PORT_USE_CLOCK:
+			case PORT_USE_SCAN:
+			    gate->direction[0] = PORT_CLASS_INPUT;
+			    break;
+
+			case PORT_USE_POWER:
+			case PORT_USE_GROUND:
+			case PORT_USE_TIEOFF:
+			case PORT_USE_ANALOG:
+			    gate->direction[0] = PORT_CLASS_BIDIRECTIONAL;
+			    break;
+		    }
+		}
 		break;
 	}
 	if (keyword == DEF_PINS_END) break;
@@ -1470,6 +1538,7 @@ DefReadComponents(FILE *f, char *sname, float oscale, int total)
 
                     gate->taps = (DSEG *)malloc(gate->nodes * sizeof(DSEG));
                     gate->noderec = (NODE *)malloc(gate->nodes * sizeof(NODE));
+                    gate->direction = (u_char *)malloc(gate->nodes * sizeof(u_char));
                     gate->netnum = (int *)malloc(gate->nodes * sizeof(int));
                     gate->node = (char **)malloc(gate->nodes * sizeof(char *));
 
@@ -1481,6 +1550,7 @@ DefReadComponents(FILE *f, char *sname, float oscale, int total)
 			/* disconnected.				*/
 
 			gate->node[i] = gateginfo->node[i];  /* copy pointer */
+			gate->direction[i] = gateginfo->direction[i];  /* copy */
 			gate->taps[i] = (DSEG)NULL;
 
 			/* Global power/ground bus check */
@@ -1932,6 +2002,7 @@ DefRead(char *inName)
 		if (sscanf(token, "%d", &total) != 1) total = 0;
 		LefEndStatement(f);
 		DefReadNets(f, sections[DEF_SPECIALNETS], oscale, TRUE, total);
+		numSpecial = total;
 		break;
 	    case DEF_NETS:
 		token = LefNextToken(f, TRUE);
