@@ -27,9 +27,9 @@ main(int argc, char *argv[])
     maskMode = MASK_AUTO;
     dofirststage(0, -1);
     maskMode = MASK_NONE;
-    result = dosecondstage(0, FALSE);
+    result = dosecondstage(0, FALSE, (u_int)100);
     if (result < 5)
-	dosecondstage(0, FALSE);
+	dosecondstage(0, FALSE, (u_int)100);
     write_def(NULL);
     return 0;
 }
