@@ -35,13 +35,14 @@ extern int     ViaCost;
 extern int     JogCost;
 extern int     XverCost;
 extern int     BlockCost;
+extern int     OffsetCost;
 extern int     ConflictCost;
 
 extern char    *ViaX[MAX_LAYERS];
 extern char    *ViaY[MAX_LAYERS];
 
 int  read_config(FILE *configfileptr, int is_info);
-void post_config();
+void post_config(void);
 
 #define QCONFIG_H
 #endif 
