@@ -20,9 +20,11 @@ qrouter_AppInit(interp)
     if (Tcl_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
-    if (Tk_Init(interp) == TCL_ERROR) {
-	return TCL_ERROR;
-    }
+
+    // Ignore Tk_Init return code---maybe can attempt to run in
+    // a non-graphics mode.
+
+    Tk_Init(interp);
     Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
 
     /* This is where we replace the home ".wishrc" file with	*/
