@@ -61,6 +61,9 @@ static int qrouter_stage3(
 static int qrouter_writedef(
     ClientData clientData, Tcl_Interp *interp,
     int objc, Tcl_Obj *CONST objv[]);
+static int qrouter_writedelays(
+    ClientData clientData, Tcl_Interp *interp,
+    int objc, Tcl_Obj *CONST objv[]);
 static int qrouter_readdef(
     ClientData clientData, Tcl_Interp *interp,
     int objc, Tcl_Obj *CONST objv[]);
@@ -136,6 +139,7 @@ static cmdstruct qrouter_commands[] =
    {"read_def", qrouter_readdef},
    {"read_lef", qrouter_readlef},
    {"read_config", qrouter_readconfig},
+   {"write_delays", qrouter_writedelays},
    {"layer_info", qrouter_layerinfo},
    {"obstruction", qrouter_obs},
    {"ignore", qrouter_ignore},
@@ -599,7 +603,7 @@ qrouter_start(ClientData clientData, Tcl_Interp *interp,
 	if (result != TCL_OK) return result;
     }
 
-    if ((DEFfilename[0] != '\0') && (Nlgates == NULL)) {
+    if ((DEFfilename != NULL) && (Nlgates == NULL)) {
 	read_def(NULL);
 	draw_layout();
     }
@@ -804,6 +808,7 @@ qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 {
     u_char dodebug;
     u_char dostep;
+    u_char saveForce, saveOverhead;
     int i, idx, idx2, val, result, failcount = 0;
     NET net = NULL;
 
@@ -826,7 +831,9 @@ qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
     dodebug = FALSE;
     dostep = FALSE;
     maskMode = MASK_AUTO;	// Mask mode is auto unless specified
-    forceRoutable = FALSE;	// Don't force unless specified
+
+    // Save these global defaults in case they are locally changed
+    saveForce = forceRoutable;
 
     if (objc >= 2) {
 	for (i = 1; i < objc; i++) {
@@ -848,7 +855,7 @@ qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 		case ForceIdx:
 		    forceRoutable = TRUE;
 		    break;
-	
+
 		case RouteIdx:
 		    if (i >= objc - 1) {
 			Tcl_WrongNumArgs(interp, 0, objv, "route ?net?");
@@ -931,6 +938,9 @@ qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 
     if (stepnet >= (Numnets - 1)) stepnet = -1;
 
+    // Restore global defaults in case they were locally changed
+    forceRoutable = saveForce;
+
     return QrouterTagCallback(interp, objc, objv);
 }
 
@@ -962,23 +972,29 @@ qrouter_stage1(ClientData clientData, Tcl_Interp *interp,
 /*  stage2 route <net>	Route net named <net> only.	*/
 /*							*/
 /*  stage2 force	Force a terminal to be routable	*/
-/*  stage2 tries <n>	Keep trying n additional times	*/
+/*  stage2 break	Only rip up colliding segment	*/
+/*  stage2 effort <n>	Level of effort (default 100)	*/
 /*------------------------------------------------------*/
 
 static int
 qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
                int objc, Tcl_Obj *CONST objv[])
 {
+    u_int  effort;
     u_char dodebug;
     u_char dostep;
+    u_char onlybreak;
+    u_char saveForce, saveOverhead;
     int i, idx, idx2, val, result, failcount;
     NET net = NULL;
 
     static char *subCmds[] = {
-	"debug", "mask", "limit", "route", "force", "tries", "step", NULL
+	"debug", "mask", "limit", "route", "force", "tries", "step",
+	"break", "effort", NULL
     };
     enum SubIdx {
-	DebugIdx, MaskIdx, LimitIdx, RouteIdx, ForceIdx, TriesIdx, StepIdx
+	DebugIdx, MaskIdx, LimitIdx, RouteIdx, ForceIdx, TriesIdx, StepIdx,
+	BreakIdx, EffortIdx
     };
    
     static char *maskSubCmds[] = {
@@ -992,9 +1008,12 @@ qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 
     dodebug = FALSE;
     dostep = FALSE;
+    onlybreak = FALSE;
     maskMode = MASK_AUTO;	// Mask mode is auto unless specified
-    forceRoutable = FALSE;	// Don't force unless specified
+    // Save these global defaults in case they are locally changed
+    saveForce = forceRoutable;
     ripLimit = 10;		// Rip limit is 10 unless specified
+    effort = 100;		// Moderate to high effort
 
     if (objc >= 2) {
 	for (i = 1; i < objc; i++) {
@@ -1012,11 +1031,26 @@ qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 		case StepIdx:
 		    dostep = TRUE;
 		    break;
+
+		case BreakIdx:
+		    onlybreak = TRUE;
+		    break;
 	
 		case ForceIdx:
 		    forceRoutable = TRUE;
 		    break;
 
+		case EffortIdx:
+		    if (i >= objc - 1) {
+			Tcl_WrongNumArgs(interp, 0, objv, "effort ?num?");
+			return TCL_ERROR;
+		    }
+		    i++;
+		    result = Tcl_GetIntFromObj(interp, objv[i], &val);
+		    if (result != TCL_OK) return result;
+		    effort = (u_int)val;
+		    break;
+
 		case TriesIdx:
 		    if (i >= objc - 1) {
 			Tcl_WrongNumArgs(interp, 0, objv, "tries ?num?");
@@ -1025,7 +1059,9 @@ qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 		    i++;
 		    result = Tcl_GetIntFromObj(interp, objv[i], &val);
 		    if (result != TCL_OK) return result;
-		    keepTrying = (u_char)val;
+		    Tcl_SetResult(interp, "\"tries\" deprecated, "
+				"use \"effort\" instead.", NULL);
+		    effort = (u_char)val * 100;
 		    break;
 	
 		case RouteIdx:
@@ -1089,12 +1125,16 @@ qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
     }
 
     if (net == NULL)
-	failcount = dosecondstage(dodebug, dostep);
+	failcount = dosecondstage(dodebug, dostep, onlybreak, effort);
     else
-	failcount = route_net_ripup(net, dodebug);
+	failcount = route_net_ripup(net, dodebug, onlybreak);
     Tcl_SetObjResult(interp, Tcl_NewIntObj(failcount));
 
     draw_layout();
+
+    // Restore global defaults in case they were locally changed
+    forceRoutable = saveForce;
+
     return QrouterTagCallback(interp, objc, objv);
 }
 
@@ -1123,22 +1163,25 @@ qrouter_stage2(ClientData clientData, Tcl_Interp *interp,
 /*  stage3 route <net>	Route net named <net> only.	*/
 /*							*/
 /*  stage3 force	Force a terminal to be routable	*/
+/*  stage3 effort	Level of effort (default 100)	*/
 /*------------------------------------------------------*/
 
 static int
 qrouter_stage3(ClientData clientData, Tcl_Interp *interp,
                int objc, Tcl_Obj *CONST objv[])
 {
+    u_int effort;
     u_char dodebug;
     u_char dostep;
+    u_char saveForce, saveOverhead;
     int i, idx, idx2, val, result, failcount = 0;
     NET net = NULL;
 
     static char *subCmds[] = {
-	"debug", "mask", "route", "force", "step", NULL
+	"debug", "mask", "route", "force", "step", "effort", NULL
     };
     enum SubIdx {
-	DebugIdx, MaskIdx, RouteIdx, ForceIdx, StepIdx
+	DebugIdx, MaskIdx, RouteIdx, ForceIdx, StepIdx, EffortIdx
     };
    
     static char *maskSubCmds[] = {
@@ -1153,7 +1196,10 @@ qrouter_stage3(ClientData clientData, Tcl_Interp *interp,
     dodebug = FALSE;
     dostep = FALSE;
     maskMode = MASK_AUTO;	// Mask mode is auto unless specified
-    forceRoutable = FALSE;	// Don't force unless specified
+    effort = 100;		// Moderate to high effort
+
+    // Save these global defaults in case they are locally changed
+    saveForce = forceRoutable;
 
     if (objc >= 2) {
 	for (i = 1; i < objc; i++) {
@@ -1175,7 +1221,18 @@ qrouter_stage3(ClientData clientData, Tcl_Interp *interp,
 		case ForceIdx:
 		    forceRoutable = TRUE;
 		    break;
-	
+
+		case EffortIdx:
+		    if (i >= objc - 1) {
+			Tcl_WrongNumArgs(interp, 0, objv, "effort ?num?");
+			return TCL_ERROR;
+		    }
+		    i++;
+		    result = Tcl_GetIntFromObj(interp, objv[i], &val);
+		    if (result != TCL_OK) return result;
+		    effort = (u_int)val;
+		    break;
+
 		case RouteIdx:
 		    if (i >= objc - 1) {
 			Tcl_WrongNumArgs(interp, 0, objv, "route ?net?");
@@ -1229,7 +1286,7 @@ qrouter_stage3(ClientData clientData, Tcl_Interp *interp,
     else stepnet++;
 
     if (net == NULL)
-	failcount = dothirdstage(dodebug, stepnet);
+	failcount = dothirdstage(dodebug, stepnet, effort);
     else {
 	if ((net != NULL) && (net->netnodes != NULL)) {
 	    result = doroute(net, (u_char)0, dodebug);
@@ -1258,6 +1315,9 @@ qrouter_stage3(ClientData clientData, Tcl_Interp *interp,
 
     if (stepnet >= (Numnets - 1)) stepnet = -1;
 
+    // Restore global defaults in case they were locally changed
+    forceRoutable = saveForce;
+
     return QrouterTagCallback(interp, objc, objv);
 }
 
@@ -1303,7 +1363,7 @@ qrouter_remove(ClientData clientData, Tcl_Interp *interp,
 	    case AllIdx:
 		for (i = 0; i < Numnets; i++) {
 		   net = Nlnets[i];
-		   ripup_net(net, (u_char)1);
+		   ripup_net(net, (u_char)1, (u_char)1);
 		}
 		draw_layout();
 		break;
@@ -1311,7 +1371,7 @@ qrouter_remove(ClientData clientData, Tcl_Interp *interp,
 		for (i = 2; i < objc; i++) {
 		    net = LookupNet(Tcl_GetString(objv[i]));
 		    if (net != NULL)
-			ripup_net(net, (u_char)1);
+			ripup_net(net, (u_char)1, (u_char)1);
 		}
 		draw_layout();
 		break;
@@ -1331,6 +1391,7 @@ qrouter_remove(ClientData clientData, Tcl_Interp *interp,
 /*			ordered by the standard metric	*/
 /*   failing unordered	Move all nets to FailedNets,	*/
 /*			as originally ordered		*/
+/*   failing summary	List of failed and total nets	*/
 /*------------------------------------------------------*/
 
 static int
@@ -1453,7 +1514,7 @@ static int
 qrouter_readdef(ClientData clientData, Tcl_Interp *interp,
                 int objc, Tcl_Obj *CONST objv[])
 {
-    if ((DEFfilename[0] == '\0') && (objc != 2)) {
+    if ((DEFfilename == NULL) && (objc != 2)) {
 	Tcl_SetResult(interp, "No DEF filename specified!", NULL);
 	return TCL_ERROR;
     }
@@ -1480,7 +1541,7 @@ qrouter_writedef(ClientData clientData, Tcl_Interp *interp,
 
     if (objc == 2)
 	DEFoutfile = Tcl_GetString(objv[1]);
-    else if (DEFfilename[0] == '\0') {
+    else if (DEFfilename == NULL) {
 	Tcl_SetResult(interp, "No DEF filename specified!", NULL);
 	return TCL_ERROR;
     }
@@ -1489,6 +1550,27 @@ qrouter_writedef(ClientData clientData, Tcl_Interp *interp,
     return QrouterTagCallback(interp, objc, objv);
 }
 
+/*------------------------------------------------------*/
+/* Command "write_delays"				*/
+/*------------------------------------------------------*/
+
+static int
+qrouter_writedelays(ClientData clientData, Tcl_Interp *interp,
+                 int objc, Tcl_Obj *CONST objv[])
+{
+    char *delayoutfile = NULL;
+
+    if (objc == 2)
+	delayoutfile = Tcl_GetString(objv[1]);
+    else if (delayfilename == NULL) {
+	Tcl_SetResult(interp, "No delay filename specified!", NULL);
+	return TCL_ERROR;
+    }
+
+    write_delays(delayoutfile);
+    return QrouterTagCallback(interp, objc, objv);
+}
+
 /*------------------------------------------------------*/
 /* Command "read_config"				*/
 /*------------------------------------------------------*/
@@ -2078,7 +2160,9 @@ qrouter_layers(ClientData clientData, Tcl_Interp *interp,
 	result = Tcl_GetIntFromObj(interp, objv[1], &value);
 	if (result != TCL_OK) return result;
 	if (value <= 0 || value > LefGetMaxLayer()) {
-	    Tcl_SetResult(interp, "Number of layers out of range", NULL);
+	    Tcl_SetResult(interp, "Number of layers out of range,"
+			" setting to max.", NULL);
+	    Num_layers = LefGetMaxLayer();
 	    return TCL_ERROR;
 	}
 	Num_layers = value;
